#!/bin/sh
#

# Command line arguments.
SECS=$1
CLIENTS=$2
PORTS=$3

# Parameter defaults.
[ -z "${SECS}"    ] && SECS=300
[ -z "${CLIENTS}" ] && CLIENTS=14
[ -z "${PORTS}"   ] && PORTS=4

# Make sure our local tools are in place.
make all > /dev/null || exit 1

# nmeter configuration.
NMETER="./jack_test3_nmeter"
NMETER_ARGS="t c i x"

# jackd configuration.
JACKD="/usr/bin/jackd"
JACKD_DRIVER="alsa"
JACKD_DEVICE="hw:0"
JACKD_PRIO=20
JACKD_RATE=44100
JACKD_PERIOD=64
JACKD_PORTS=$(((${CLIENTS} + 1) * ${PORTS} * 2))
JACKD_ARGS="-v -R -P${JACKD_PRIO} -p${JACKD_PORTS} -d${JACKD_DRIVER} -d${JACKD_DEVICE} -r${JACKD_RATE} -p${JACKD_PERIOD} -n2 -S -P"

# client test configuration.
CLIENT="./jack_test3_client"
CLIENT_ARGS="${SECS} ${PORTS}"

# process/thread list configuration.
PIDOF="/sbin/pidof"
PLIST="ps -mo pid,tid,class,rtprio,ni,pri,pcpu,stat,comm"

# Log filename.
LOG="jack_test3-`uname -r`-`date +'%Y%m%d%H%M'`.log"

# Command executive and logger.
exec_log ()
{
	CMD="$*"
	echo "-----------------------"		>> ${LOG} 2>&1
	echo "# ${CMD}"				>> ${LOG} 2>&1
	${CMD}					>> ${LOG} 2>&1
	echo					>> ${LOG} 2>&1
}

# Process/thread status logger.
plist_log ()
{
	PIDS="$*"
	if [ -n "${PIDS}" ]; then
		echo "- - - - - - - - - - - -"	>> ${LOG} 2>&1
		${PLIST} ${PIDS}		>> ${LOG} 2>&1
		echo				>> ${LOG} 2>&1
	fi
}

# IRQ handler process/thread status logger.
show_irq ()
{
	IRQ=$1
	plist_log `${PIDOF} "IRQ ${IRQ}"`
}

show_irq_name ()
{
	NAME=$1
	IRQS=`cat /proc/interrupts | grep "${NAME}" | awk -F: '{print $1}'`
	for IRQ in ${IRQS}; do
		show_irq ${IRQ}
	done
}


#
# Log headings -- show some relevant info...
#
echo "*** Started `date` ***" >> ${LOG} 2>&1

echo >> ${LOG} 2>&1
echo "Seconds to run        (SECS) = ${SECS}"		>> ${LOG} 2>&1
echo "Number of clients  (CLIENTS) = ${CLIENTS}"	>> ${LOG} 2>&1
echo "Ports per client     (PORTS) = ${PORTS}"		>> ${LOG} 2>&1
echo >> ${LOG} 2>&1

exec_log "uname -a"

exec_log "cat /proc/asound/version"
exec_log "cat /proc/asound/cards"

#exec_log "grep . /proc/sys/kernel/*_preemption"
#exec_log "grep . /proc/irq/*/*/threaded"
#exec_log "grep . /sys/block/hd*/queue/max_sectors_kb"

exec_log "cat /proc/interrupts"

# This is just about to be sure...
#plist_log -C ksoftirqd/0

# Keyboard/Mouse IRQs (normal is 1 and 12).
#show_irq_name "i8042"

# Real time clock IRQs (normal is 8).
#show_irq_name "rtc"

# USB controllers IRQs (normal is 10).
#show_irq_name "ohci_hcd"

# Soundcard IRQs (normal is 5).
SND_IRQS=`grep irq /proc/asound/cards | sed 's/.* irq \(.*\)/\1/'`
for SND_IRQ in ${SND_IRQS}; do
	show_irq ${SND_IRQ}
done

#
# Lauch nmeter in the background...
#
echo -n "Launching `basename ${NMETER}`..."
(${NMETER} ${NMETER_ARGS} >> ${LOG} 2>&1) &
sleep 2
echo "done."
sleep 1

# Lauch the test client suite...
SLEEP=6
echo -n "Launching ${CLIENTS} ${CLIENT}(s) instance(s)..."
for NUM in `seq 1 ${CLIENTS}`; do
	CLIENT_CMDLINE="${CLIENT} ${CLIENT_ARGS}"
	SLEEP=$((${SLEEP} + 3))
	(sleep ${SLEEP}; echo -n "$NUM..."; exec_log ${CLIENT_CMDLINE}) &
done
echo "done."

# Let there be some evidence of process status...
SLEEP=$((${SLEEP} + 6))
(sleep ${SLEEP}; plist_log -C `basename ${JACKD}`; plist_log -C `basename ${CLIENT}`) &

# Let jackd live for extended but limited time...
SLEEP=$((${SLEEP} + ${SECS}))
(sleep ${SLEEP}; killall `basename ${JACKD}`) &

#
# Lauch jackd and wait for it...
#
echo -n "Running `basename ${JACKD}`..."
exec_log ${JACKD} ${JACKD_ARGS}
echo "done."
sleep 3

#echo -n "Killing `basename ${CLIENT}`..."
#killall `basename ${CLIENT}` && echo "OK." || echo "FAILED."
#sleep 1

echo -n "Killing `basename ${NMETER}`..."
killall `basename ${NMETER}` && echo "OK." || echo "FAILED."

echo "*** Terminated `date` ***" >> ${LOG} 2>&1

sync
sleep 1
sync

# Summary log analynis...
cat ${LOG} | awk -f jack_test3_summary.awk | tee -a ${LOG}
